@@ -70,7 +70,7 @@ on_about_activate (GtkMenuItem *menuitem, gpointer user_data)
 		"Fabio Marzocca <thesaltydog@gmail.com>",
 		"Paolo Borelli <pborelli@katamail.com>",
 		"Benoît Dejean <benoit@placenet.org>",
-		"Igalia (rings-chart widget) <www.igalia.com>",
+		"Igalia (rings-chart and treemap widget) <www.igalia.com>",
 		NULL
 	};
 	
