@@ -25,9 +25,9 @@
 /* MAYBE I LIED... -jrb */
 
 #include <config.h>
-#include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <gdk/gdkx.h>
+#include <gdk/gdkkeysyms.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -55,7 +55,6 @@
 #define LAST_SAVE_DIRECTORY_KEY GNOME_SCREENSHOT_GCONF "/last_save_directory"
 #define BORDER_EFFECT_KEY       GNOME_SCREENSHOT_GCONF "/border_effect"
 #define DELAY_KEY               GNOME_SCREENSHOT_GCONF "/delay"
-#define TAKE_WINDOW_SHOT_KEY    GNOME_SCREENSHOT_GCONF "/take_window_shot"
 
 
 enum
@@ -87,6 +86,7 @@ typedef struct
   int iteration;
   TestType type;
   GdkWindow *window;
+  GdkRectangle *rectangle;
 } AsyncExistenceJob;
 
 static GdkPixbuf *screenshot = NULL;
@@ -99,6 +99,7 @@ static gboolean save_immediately = FALSE;
 
 /* Options */
 static gboolean take_window_shot = FALSE;
+static gboolean take_area_shot = FALSE;
 static gboolean include_border = FALSE;
 static gboolean include_pointer = TRUE;
 static char *border_effect = NULL;
@@ -113,23 +114,23 @@ static void  save_options           (void);
 static GtkWidget *border_check = NULL;
 static GtkWidget *effect_combo = NULL;
 static GtkWidget *effect_label = NULL;
+static GtkWidget *delay_hbox = NULL;
 
 static void
 display_help (GtkWindow *parent)
 {
   GError *error = NULL;
 
-  gnome_help_display_desktop (NULL,
-                              "user-guide",
-			      "user-guide.xml",
-                              "goseditmainmenu-53",
-			      &error);
+  gtk_show_uri (gtk_window_get_screen (parent),
+		"ghelp:user-guide#goseditmainmenu-53",
+		gtk_get_current_event_time (), &error);
 
   if (error)
     {
       screenshot_show_gerror_dialog (parent,
                                      _("Error loading the help page"),
                                      error);
+      g_error_free (error);
     }
 }
 
@@ -150,19 +151,26 @@ interactive_dialog_response_cb (GtkDialog *dialog,
     }
 }
 
+#define TARGET_TOGGLE_DESKTOP 0
+#define TARGET_TOGGLE_WINDOW  1
+#define TARGET_TOGGLE_AREA    2
+
 static void
 target_toggled_cb (GtkToggleButton *button,
                    gpointer         data)
 {
-  gboolean window_selected = (GPOINTER_TO_INT (data) == TRUE ? TRUE : FALSE);
+  int target_toggle = GPOINTER_TO_INT (data);
 
   if (gtk_toggle_button_get_active (button))
     {
-      take_window_shot = window_selected;
+      take_window_shot = (target_toggle == TARGET_TOGGLE_WINDOW);
+      take_area_shot = (target_toggle == TARGET_TOGGLE_AREA);
       
       gtk_widget_set_sensitive (border_check, take_window_shot);
       gtk_widget_set_sensitive (effect_combo, take_window_shot);
       gtk_widget_set_sensitive (effect_label, take_window_shot);
+
+      gtk_widget_set_sensitive (delay_hbox, !take_area_shot);
     }
 }
 
@@ -423,11 +431,11 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
   group = NULL;
   radio = gtk_radio_button_new_with_mnemonic (group,
                                               _("Grab the whole _desktop"));
-  if (take_window_shot)
+  if (take_window_shot || take_area_shot)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), FALSE);
   g_signal_connect (radio, "toggled",
                     G_CALLBACK (target_toggled_cb),
-                    GINT_TO_POINTER (FALSE));
+                    GINT_TO_POINTER (TARGET_TOGGLE_DESKTOP));
   gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 0);
   group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
   gtk_widget_show (radio);
@@ -439,32 +447,44 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);
   g_signal_connect (radio, "toggled",
                     G_CALLBACK (target_toggled_cb),
-                    GINT_TO_POINTER (TRUE));
+                    GINT_TO_POINTER (TARGET_TOGGLE_WINDOW));
+  gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 0);
+  group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
+  gtk_widget_show (radio);
+
+  /** Grab area of the desktop **/
+  radio = gtk_radio_button_new_with_mnemonic (group,
+                                              _("Grab a selected _area"));
+  if (take_area_shot)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);
+  g_signal_connect (radio, "toggled",
+                    G_CALLBACK (target_toggled_cb),
+                    GINT_TO_POINTER (TARGET_TOGGLE_AREA));
   gtk_box_pack_start (GTK_BOX (vbox), radio, FALSE, FALSE, 0);
   gtk_widget_show (radio);
 
   /** Grab after delay **/
-  hbox = gtk_hbox_new (FALSE, 6);
-  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-  gtk_widget_show (hbox);
+  delay_hbox = gtk_hbox_new (FALSE, 6);
+  gtk_box_pack_start (GTK_BOX (vbox), delay_hbox, FALSE, FALSE, 0);
+  gtk_widget_show (delay_hbox);
 
   /* translators: this is the first part of the "grab after a
    * delay of <spin button> seconds".
    */
   label = gtk_label_new_with_mnemonic (_("Grab _after a delay of"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (delay_hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
   adjust = GTK_ADJUSTMENT (gtk_adjustment_new ((gdouble) delay,
                                                0.0, 99.0,
                                                1.0,  1.0,
-                                               1.0));
+                                               0.0));
   spin = gtk_spin_button_new (adjust, 1.0, 0);
   g_signal_connect (spin, "value-changed",
                     G_CALLBACK (delay_spin_value_changed_cb),
                     NULL);
-  gtk_box_pack_start (GTK_BOX (hbox), spin, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (delay_hbox), spin, FALSE, FALSE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), spin);
   gtk_widget_show (spin);
 
@@ -473,7 +493,7 @@ create_screenshot_frame (GtkWidget   *outer_vbox,
    */
   label = gtk_label_new (_("seconds"));
   gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-  gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (delay_hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 }
 
@@ -487,7 +507,7 @@ create_interactive_dialog (void)
   gtk_window_set_resizable (GTK_WINDOW (retval), FALSE);
   gtk_container_set_border_width (GTK_CONTAINER (retval), 5);
   gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (retval)->vbox), 2);
-  gtk_window_set_title (GTK_WINDOW (retval), "");
+  gtk_window_set_title (GTK_WINDOW (retval), _("Take Screenshot"));
 
   /* main container */
   main_vbox = gtk_vbox_new (FALSE, 18);
@@ -537,9 +557,48 @@ save_folder_to_gconf (ScreenshotDialog *dialog)
 			   LAST_SAVE_DIRECTORY_KEY, folder,
                            NULL);
 
+  g_free (folder);
   g_object_unref (gconf_client);
 }
 
+static void
+set_recent_entry (ScreenshotDialog *dialog)
+{
+  char *uri, *app_exec = NULL;
+  GtkRecentManager *recent;
+  GtkRecentData recent_data;
+  GAppInfo *app;
+  const char *exec_name = NULL;
+  static char * groups[2] = { "Graphics", NULL };
+
+  app = g_app_info_get_default_for_type ("image/png", TRUE);
+
+  if (!app) {
+    /* return early, as this would be an useless recent entry anyway. */
+    return;
+  }
+
+  uri = screenshot_dialog_get_uri (dialog);
+  recent = gtk_recent_manager_get_default ();
+  
+  exec_name = g_app_info_get_executable (app);
+  app_exec = g_strjoin (" ", exec_name, "%u", NULL);
+
+  recent_data.display_name = NULL;
+  recent_data.description = NULL;
+  recent_data.mime_type = "image/png";
+  recent_data.app_name = "GNOME Screenshot";
+  recent_data.app_exec = app_exec;
+  recent_data.groups = groups;
+  recent_data.is_private = FALSE;
+
+  gtk_recent_manager_add_full (recent, uri, &recent_data);
+
+  g_object_unref (app);
+  g_free (app_exec);
+  g_free (uri);
+}
+
 static void
 error_dialog_response_cb (GtkDialog *d,
                           gint response,
@@ -564,6 +623,7 @@ save_callback (TransferResult result,
   if (result == TRANSFER_OK)
     {
       save_folder_to_gconf (dialog);
+      set_recent_entry (dialog);
       gtk_widget_destroy (toplevel);
       
       /* we're done, stop the mainloop now */
@@ -595,7 +655,7 @@ save_callback (TransferResult result,
       /* translators: first %s is the file path, second %s is the VFS error */
       gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
                                                 _("Impossible to save the screenshot "
-                                                  "to %s.\n Error was %s\n. Please choose another "
+                                                  "to %s.\n Error was %s.\n Please choose another "
                                                   "location and retry."), uri, error_message);
       gtk_widget_show (error_dialog);
       g_signal_connect (error_dialog,
@@ -710,26 +770,30 @@ run_dialog (ScreenshotDialog *dialog)
 }
 
 static void
-finish_prepare_screenshot (char *initial_uri, GdkWindow *window)
+finish_prepare_screenshot (char *initial_uri, GdkWindow *window, GdkRectangle *rectangle)
 {  
   ScreenshotDialog *dialog;
 
-  screenshot = screenshot_get_pixbuf (window, include_pointer, include_border);
+  /* always disable window border for full-desktop or selected-area screenshots */
+  if (!take_window_shot)
+    screenshot = screenshot_get_pixbuf (window, rectangle, include_pointer, FALSE);
+  else
+    {
+      screenshot = screenshot_get_pixbuf (window, rectangle, include_pointer, include_border);
 
-  if (take_window_shot) {
-    switch (border_effect[0])
-      {
-      case 's': /* shadow */
-        screenshot_add_shadow (&screenshot);
-        break;
-      case 'b': /* border */
-        screenshot_add_border (&screenshot);
-        break;
-      case 'n': /* none */
-      default:
-        break;
-      }
-  }
+      switch (border_effect[0])
+        {
+        case 's': /* shadow */
+          screenshot_add_shadow (&screenshot);
+          break;
+        case 'b': /* border */
+          screenshot_add_border (&screenshot);
+          break;
+        case 'n': /* none */
+        default:
+          break;
+        }
+    }
 
   /* release now the lock, it was acquired when we were finding the window */
   screenshot_release_lock ();
@@ -750,19 +814,26 @@ finish_prepare_screenshot (char *initial_uri, GdkWindow *window)
   run_dialog (dialog);
 }
 
+static void
+async_existence_job_free (AsyncExistenceJob *job)
+{
+  if (!job)
+    return;
+
+  g_free (job->base_uris[1]);
+  g_free (job->base_uris[2]);
+  g_free (job->rectangle);
+  g_slice_free (AsyncExistenceJob, job);
+}
+
 static gboolean
 check_file_done (gpointer user_data)
 {
-  char *retval;
-  GdkWindow *window;
   AsyncExistenceJob *job = user_data;
 
-  window = job->window;
-  retval = job->retval;
-  g_free (job->base_uris[1]);
-  g_slice_free (AsyncExistenceJob, job);
-  
-  finish_prepare_screenshot (retval, window);
+  finish_prepare_screenshot (job->retval, job->window, job->rectangle);
+
+  async_existence_job_free (job);
   
   return FALSE;
 }
@@ -808,7 +879,7 @@ build_uri (AsyncExistenceJob *job)
 
   retval = g_build_filename (job->base_uris[job->type], file_name, NULL);
   g_free (file_name);
-  
+
   return retval;
 }
 
@@ -835,7 +906,7 @@ retry:
       /* file already exists, iterate again */
       g_object_unref (info);
       g_object_unref (file);
-      g_error_free (error);
+      g_free (uri);
 
       (job->iteration)++;
 
@@ -848,8 +919,30 @@ retry:
        */
       if (error->code == G_IO_ERROR_NOT_FOUND)
         {
-          job->retval = uri;
-          goto out;
+          GFile *parent;
+
+          /* if the parent directory doesn't exist as well, forget the saved
+           * directory and treat this as a generic error.
+           */
+
+          parent = g_file_get_parent (file);
+
+          if (!g_file_query_exists (parent, NULL))
+            {
+              (job->type)++;
+              job->iteration = 0;
+
+              g_object_unref (file);
+              g_object_unref (parent);
+              goto retry;
+            }
+          else
+            {
+              job->retval = uri;
+
+              g_object_unref (parent);
+              goto out;
+            }
         }
       else
         {
@@ -919,6 +1012,30 @@ find_current_window (char **window_title)
   return window;
 }
 
+static GdkRectangle *
+find_rectangle (void)
+{
+  GdkRectangle *rectangle;
+
+  if (!take_area_shot)
+    return NULL;
+
+  rectangle = g_new0 (GdkRectangle, 1);
+  if (screenshot_select_area (&rectangle->x, &rectangle->y,
+                              &rectangle->width, &rectangle->height))
+    {
+      g_assert (rectangle->width >= 0);
+      g_assert (rectangle->height >= 0);
+
+      return rectangle;
+    }
+  else
+    {
+      g_free (rectangle);
+      return NULL;
+    }
+}
+
 static void
 prepare_screenshot (void)
 {
@@ -926,12 +1043,22 @@ prepare_screenshot (void)
   
   job = g_slice_new0 (AsyncExistenceJob);
   job->base_uris[0] = last_save_dir;
-  /* we'll have to free this */
+  /* we'll have to free these two */
   job->base_uris[1] = get_desktop_dir ();
-  job->base_uris[2] = (char *) g_get_tmp_dir ();
+  job->base_uris[2] = g_strconcat ("file://", g_get_tmp_dir (), NULL);
   job->iteration = 0;
   job->type = TEST_LAST_DIR;
   job->window = find_current_window (&window_title);
+  job->rectangle = find_rectangle ();
+
+  /* Check if the area selection was cancelled */
+  if (job->rectangle &&
+      (job->rectangle->width == 0 || job->rectangle->height == 0))
+    {
+      async_existence_job_free (job);
+      gtk_main_quit ();
+      return;
+    }
 
   g_io_scheduler_push_job (try_check_file,
                            job,
@@ -962,9 +1089,9 @@ get_desktop_dir (void)
                                                "/apps/nautilus/preferences/desktop_is_home_dir",
                                                NULL);
   if (desktop_is_home_dir)
-    desktop_dir = g_build_filename (g_get_home_dir (), NULL);
+    desktop_dir = g_strconcat ("file://", g_get_home_dir (), NULL);
   else
-    desktop_dir = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
+    desktop_dir = g_strconcat ("file://", g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP), NULL);
 
   g_object_unref (gconf_client);
 
@@ -1005,12 +1132,8 @@ expand_initial_tilde (const char *path)
 static void
 load_options (void)
 {
-  GnomeClient *client;
   GConfClient *gconf_client;
 
-  client = gnome_master_client ();
-  gnome_client_set_restart_style (client, GNOME_RESTART_NEVER);
-
   gconf_client = gconf_client_get_default ();
 
   /* Find various dirs */
@@ -1042,12 +1165,6 @@ load_options (void)
   if (!border_effect)
     border_effect = g_strdup ("none");
 
-#if 0
-  take_window_shot = gconf_client_get_bool (gconf_client,
-                                            TAKE_WINDOW_SHOT_KEY,
-                                            NULL);
-#endif
-
   delay = gconf_client_get_int (gconf_client, DELAY_KEY, NULL);
 
   g_object_unref (gconf_client);
@@ -1062,19 +1179,12 @@ save_options (void)
 
   /* Error is NULL, as there's nothing we can do */
 
-#if 0
-  /* Disable saving the window key as it breaks command line
-   * and global key binding
-   */
-  gconf_client_set_bool (gconf_client,
-                         TAKE_WINDOW_SHOT_KEY,
-                         take_window_shot,
-                         NULL);
-#endif
-
   gconf_client_set_bool (gconf_client,
                          INCLUDE_BORDER_KEY, include_border,
                          NULL);
+  gconf_client_set_bool (gconf_client,
+                         INCLUDE_POINTER_KEY, include_pointer,
+                         NULL);
   gconf_client_set_int (gconf_client, DELAY_KEY, delay, NULL);
   gconf_client_set_string (gconf_client,
                            BORDER_EFFECT_KEY, border_effect,
@@ -1117,20 +1227,22 @@ screenshooter_init_stock_icons (void)
 int
 main (int argc, char *argv[])
 {
-  GnomeProgram *program;
   GOptionContext *context;
   GOptionGroup *group;
   gboolean window_arg = FALSE;
+  gboolean area_arg = FALSE;
   gboolean include_border_arg = FALSE;
   gboolean disable_border_arg = FALSE;
   gboolean interactive_arg = FALSE;
   gchar *border_effect_arg = NULL;
   guint delay_arg = 0;
+  GError *error = NULL;
 
   const GOptionEntry entries[] = {
     { "window", 'w', 0, G_OPTION_ARG_NONE, &window_arg, N_("Grab a window instead of the entire screen"), NULL },
+    { "area", 'a', 0, G_OPTION_ARG_NONE, &area_arg, N_("Grab an area of the screen instead of the entire screen"), NULL },
     { "include-border", 'b', 0, G_OPTION_ARG_NONE, &include_border_arg, N_("Include the window border with the screenshot"), NULL },
-    { "remove-border", 'B', 0, G_OPTION_ARG_NONE, &disable_border_arg, "Remove the window border from the screenshot", NULL },
+    { "remove-border", 'B', 0, G_OPTION_ARG_NONE, &disable_border_arg, N_("Remove the window border from the screenshot"), NULL },
     { "delay", 'd', 0, G_OPTION_ARG_INT, &delay_arg, N_("Take screenshot after specified delay [in seconds]"), N_("seconds") },
     { "border-effect", 'e', 0, G_OPTION_ARG_STRING, &border_effect_arg, N_("Effect to add to the border (shadow, border or none)"), N_("effect") },
     { "interactive", 'i', 0, G_OPTION_ARG_NONE, &interactive_arg, N_("Interactively set options"), NULL },
@@ -1142,17 +1254,31 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  g_thread_init (NULL);
+
   context = g_option_context_new (_("Take a picture of the screen"));
   g_option_context_set_ignore_unknown_options (context, FALSE);
   g_option_context_set_help_enabled (context, TRUE);
   g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
+  g_option_context_add_group (context, gtk_get_option_group (TRUE));
+
+  g_option_context_parse (context, &argc, &argv, &error);
+
+  if (error) {
+    g_critical ("Unable to parse arguments: %s", error->message);
+    g_error_free (error);
+    g_option_context_free (context);
+    exit (1);
+  }
+
+  g_option_context_free (context);
+
+  if (window_arg && area_arg) {
+    g_printerr (_("Conflicting options: --window and --area should not be "
+                  "used at the same time.\n"));
+    exit (1);
+  }
 
-  program = gnome_program_init ("gnome-screenshot", VERSION,
-				LIBGNOMEUI_MODULE,
-				argc, argv,
-				GNOME_PARAM_GOPTION_CONTEXT, context,
-				GNOME_PROGRAM_STANDARD_PROPERTIES,
-				NULL);
   gtk_window_set_default_icon_name (SCREENSHOOTER_ICON);
   screenshooter_init_stock_icons ();
 
@@ -1161,6 +1287,9 @@ main (int argc, char *argv[])
   if (window_arg)
     take_window_shot = TRUE;
 
+  if (area_arg)
+    take_area_shot = TRUE;
+
   if (include_border_arg)
     include_border = TRUE;
 
@@ -1190,7 +1319,6 @@ main (int argc, char *argv[])
         {
         case GTK_RESPONSE_DELETE_EVENT:
         case GTK_RESPONSE_CANCEL:
-          g_object_unref (program);
           return EXIT_SUCCESS;
         case GTK_RESPONSE_OK:
           break;
@@ -1200,7 +1328,8 @@ main (int argc, char *argv[])
         }
     }
 
-  if (delay > 0)
+  if (((delay > 0 && interactive_arg) || delay_arg > 0) &&
+      !take_area_shot)
     {      
       g_timeout_add (delay * 1000,
 		     prepare_screenshot_timeout,
@@ -1214,7 +1343,5 @@ main (int argc, char *argv[])
 
   gtk_main ();
 
-  g_object_unref (program);
-
   return EXIT_SUCCESS;
 }
